//
// LDASTools Frame Utilities - A library to extend manipulation of LIGO/Virgo
// Frame Files
//
// Copyright (C) 2018 California Institute of Technology
//
// LDASTools Frame Utilities is free software; you may redistribute it and/or
// modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 (GPLv2) of the
// License or at your discretion, any later version.
//
// LDASTools Frame Utilities is distributed in the hope that it will be useful,
// but without any warranty or even the implied warranty of merchantability or
// fitness for a particular purpose. See the GNU General Public License (GPLv2)
// for more details.
//
// Neither the names of the California Institute of Technology (Caltech),
// The Massachusetts Institute of Technology (M.I.T), The Laser
// Interferometer Gravitational-Wave Observatory (LIGO), nor the names
// of its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// You should have received a copy of the licensing terms for this
// software included in the file LICENSE located in the top-level
// directory of this package. If you did not, you can view a copy at
// http://dcc.ligo.org/M1500244/LICENSE
//

#ifndef FRAME_API__CREATE_RDS_HH
#define FRAME_API__CREATE_RDS_HH

#include "ldastoolsal/types.hh"
#if !defined( SWIGIMPORTED )
#include "frameAPI/rdsframe.hh"
#include "frameAPI/rdsresample.hh"
#include "frameAPI/Frame.hh"
#endif /* !defined( SWIGIMPORTED ) */

#define CREATERDS_START_TYPE INT_4U
#define CREATERDS_END_TYPE INT_4U

#if !defined( SWIGIMPORTED )
namespace FrameAPI
{
    typedef ::RDSFrame::frame_file_container_type frame_file_container_type;
    typedef struct
    {
        typedef ::RDSFrame::channel_container_type channel_type;
        typedef std::vector< INT_2U >              resampling_type;

        channel_type    names;
        resampling_type resampling;
    } channel_container_type;

    typedef CREATERDS_START_TYPE                  start_type;
    typedef CREATERDS_END_TYPE                    end_type;
    typedef RDS::Options::compression_level_type  compression_level_type;
    typedef RDS::Options::compression_method_type compression_method_type;
    typedef RDS::Options::frames_per_file_type    frames_per_file_type;
    typedef RDS::Options::seconds_per_frame_type  seconds_per_frame_type;

    Frame createRDSFrame( const frame_file_container_type& FrameFiles,
                          start_type                       Start,
                          end_type                         End,
                          const channel_container_type&    Channels );

    void createRDSSet( const frame_file_container_type&  FrameFiles,
                       const channel_container_type&     Channels,
                       const FrameAPI::RDS::FileOptions& Opts );
} // namespace FrameAPI
#endif /* !defined( SWIGIMPORTED ) */

#endif /* FRAME_API__CREATE_RDS_HH */
